package cn.estsh.i3plus.pojo.base.jpa.daoimpl;

import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.jpa.dao.BaseRepository;
import cn.estsh.i3plus.pojo.base.codemaker.SnowflakeIdMaker;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.jpa.repository.support.SimpleJpaRepository;

import javax.persistence.EntityManager;
import javax.persistence.Id;
import javax.persistence.Query;
import java.io.Serializable;
import java.lang.reflect.Field;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @Description : 自定义持久化接口实现
 * @Reference :
 * @Author : alwaysfrin
 * @CreateDate : 2018-09-13 11:24
 * @Modify:
 **/
public class BaseRepositoryImpl<T, ID extends Serializable> extends SimpleJpaRepository<T, ID>
                                                            implements BaseRepository<T, ID> {
    public static final Logger LOGGER = LoggerFactory.getLogger(BaseRepositoryImpl.class);

    /**
     * 持久化上下文
     */
    private final EntityManager entityManager;

    private Class<T> persistentClass;
    private SnowflakeIdMaker snowflakeIdMaker;

    public BaseRepositoryImpl(Class<T> clz, EntityManager em) {
        super(clz, em);
        this.entityManager = em;
        this.persistentClass = clz;
        this.snowflakeIdMaker = new SnowflakeIdMaker();
    }

    private void setParameter(Query query, String[] propName, Object[] propValue) {
        String name = null;
        Object value = null;
        for (int i = 0; i < propName.length; i++) {
            name = propName[i];
            value = propValue[i];
            if (value != null) {
                if ((value instanceof Object[])) {
                    query.setParameter(name.replace(".", ""), (Object[]) value);
                } else if ((value instanceof Collection)) {
                    query.setParameter(name.replace(".", ""), (Collection) value);
                } else {
                    query.setParameter(name.replace(".", ""), value);
                }
            }
        }
    }

    private void appendQL(StringBuffer sb, String[] propName, Object[] propValue) {
        String name = null;
        Object value = null;
        for (int i = 0; i < propName.length; i++) {
            name = propName[i];
            value = propValue[i];
            if (((value instanceof Object[])) || ((value instanceof Collection))) {
                Object[] arraySerializable = (Object[]) value;
                if ((arraySerializable != null) && (arraySerializable.length > 0)) {
                    sb.append(" and model." + name + " in (:" + name.replace(".", "") + ")");
                }
            } else if (value == null) {
                sb.append(" and model." + name + " is null ");
            } else {
                sb.append(" and model." + name + "=:" + name.replace(".", ""));
            }
        }
    }

    @Override
    public T insert(T item) {
        return innerSave(item);
    }

    @Override
    public <S extends T> S save(S entity) {
        //复写save方法，若id为0或空则新增，不然则修改
        return (S) innerSave(entity);
    }

    @Override
    public void update(Object... item) {
        if (null != item) {
            for (Object entity : item) {
                entityManager.merge(entity);
            }
        }
    }

    @Override
    public int executeUpdate(String qlString, Object... values) {
        Query query = entityManager.createQuery(qlString);
        if (values != null) {
            for (int i = 0; i < values.length; i++) {
                query.setParameter(i + 1, values[i]);
            }
        }
        return query.executeUpdate();
    }

    @Override
    public int executeUpdate(String qlString, Map<String, Object> params) {
        Query query = entityManager.createQuery(qlString);
        for (String name : params.keySet()) {
            query.setParameter(name, params.get(name));
        }
        return query.executeUpdate();
    }

    @Override
    public int executeUpdate(String qlString, List<Object> values) {
        Query query = entityManager.createQuery(qlString);
        for (int i = 0; i < values.size(); i++) {
            query.setParameter(i + 1, values.get(i));
        }
        return query.executeUpdate();
    }

    @Override
    public void deleteById(ID id){
        deleteByProperty("id", id);
    }

    @Override
    public int deleteByProperty(String propName, Object propValue) {
        return deleteByProperties(new String[] { propName }, new Object[] { propValue });
    }

    @Override
    public int deleteByProperties(String[] propName, Object[] propValue) {
        if ((propName != null) && (propName.length > 0) && (propValue != null) && (propValue.length > 0) && (propValue.length == propName.length)) {
            StringBuffer sb = new StringBuffer("delete from " + persistentClass.getName() + " model where 1=1 ");
            appendQL(sb,propName,propValue);
            Query query = entityManager.createQuery(sb.toString());
            setParameter(query,propName,propValue);

            return query.executeUpdate();
        }else{
            throw new IllegalArgumentException("删除错误!propName:" + propName + ",propValue:" + propValue);
        }
    }

    @Override
    public int deleteByIds(ID[] ids) {
        return deleteByPropertyIn("id",  ids);
    }

    @Override
    public int deleteByPropertyIn(String propName, Object[] propValues) {
        if ((propName != null && propName.length() > 0) && (propValues != null && propValues.length > 0)) {
            String hql = "delete from " + persistentClass.getName() + " model where model."+propName+" in(:"+propName+") ";
            Query query = entityManager.createQuery(hql);
            query.setParameter(propName, Arrays.asList(propValues));

            return query.executeUpdate();
        }else{
            throw new IllegalArgumentException("删除出错："+propName+":" + propValues);
        }
    }

    @Override
    public int updateByProperties(String conditionName, Object conditionValue, String propertyName, Object propertyValue) {
        return updateByProperties(new String[] { conditionName }, new Object[] { conditionValue }, new String[] { propertyName }, new Object[] { propertyValue });
    }

    @Override
    public int updateByProperties(String conditionName, Object conditionValue, String[] propertyName, Object[] propertyValue) {
        return updateByProperties(new String[] { conditionName }, new Object[] { conditionValue }, propertyName, propertyValue);
    }

    @Override
    public int updateByProperties(String[] conditionName, Object[] conditionValue, String propertyName, Object propertyValue) {
        return updateByProperties(conditionName, conditionValue, new String[] { propertyName }, new Object[] { propertyValue });
    }

    @Override
    public int updateByProperties(String[] conditionName, Object[] conditionValue, String[] propertyName, Object[] propertyValue) {
        if ((propertyName != null) && (propertyName.length > 0) && (propertyValue != null)
                && (propertyValue.length > 0) && (propertyName.length == propertyValue.length)
                && (conditionValue != null) && (conditionValue.length > 0)) {
            StringBuffer sb = new StringBuffer();

            sb.append("update " + persistentClass.getName() + " model set ");
            for (int i = 0; i < propertyName.length; i++) {
                sb.append(propertyName[i] + " = :p_" + propertyName[i] + ",");
            }
            sb.deleteCharAt(sb.length() - 1);

            sb.append(" where 1=1 ");
            appendQL(sb, conditionName, conditionValue);
            Query query = entityManager.createQuery(sb.toString());
            for (int i = 0; i < propertyName.length; i++) {
                query.setParameter("p_" + propertyName[i], propertyValue[i]);
            }

            setParameter(query, conditionName, conditionValue);
            return query.executeUpdate();
        } else {
            throw new IllegalArgumentException("参数值错误!conditionName:" + conditionName
                    + ",conditionValue:" + conditionValue + ",propertyName:" + propertyName + ",propertyValue:" + propertyValue);
        }
    }

    @Override
    public int updateByHqlWhere(String hqlWhere, String propertyName, Object propertyValue) {
        return updateByHqlWhere(hqlWhere, new String[]{propertyName}, new Object[]{propertyValue});
    }

    @Override
    public int updateByHqlWhere(String hqlWhere, String[] propertyName, Object[] propertyValue) {
        if ((propertyName != null) && (propertyName.length > 0) && (propertyValue != null)
                && (propertyValue.length > 0) && (propertyName.length == propertyValue.length)) {
            StringBuffer sb = new StringBuffer();

            sb.append("update " + persistentClass.getName() + " model set ");
            for (int i = 0; i < propertyName.length; i++) {
                sb.append(propertyName[i] + " = :p_" + propertyName[i] + ",");
            }
            sb.deleteCharAt(sb.length() - 1);

            sb.append(" where 1=1 ");
            sb.append(hqlWhere);
            Query query = entityManager.createQuery(sb.toString());
            for (int i = 0; i < propertyName.length; i++) {
                query.setParameter("p_" + propertyName[i], propertyValue[i]);
            }

            return query.executeUpdate();
        } else {
            throw new IllegalArgumentException("参数值错误! propertyName:" + propertyName + ",propertyValue:" + propertyValue);
        }
    }

    @Override
    public T getById(long id) {
        try{
            return entityManager.find(persistentClass,id);
        }catch (Exception e){
            return null;
        }
    }

    @Override
    public List<T> list() {
        return this.findAll();
    }

    @Override
    public List<T> listPager(Pager pager) {
        return entityManager.createQuery("from " + persistentClass.getName())
                .setFirstResult(pager.getStartRow())
                .setMaxResults(pager.getPageSize())
                .getResultList();
    }

    @Override
    public List<T> list(final String hql) {
        return entityManager.createQuery(hql).getResultList();
    }

    @Override
    public List<T> findByPage(String hql, int offset, int pageSize) {
        return entityManager.createQuery("from " + persistentClass.getName())
                .setFirstResult(offset)
                .setMaxResults(pageSize)
                .getResultList();
    }

    @Override
    public List<T> findByPage(String hql, Object[] values, int offset, int pageSize) {
        Query query = entityManager.createQuery(hql);
        for (int i = 0; i < values.length; i++) {
            query.setParameter(i, values[i]);
        }
        return query.setFirstResult(offset).setMaxResults(pageSize).getResultList();
    }

    @Override
    public List<T> findByProperty(String propertyName, Object value) {
        String queryString = "from " + persistentClass.getSimpleName() + " as model where model." + propertyName + "= :" + propertyName;
        return entityManager.createQuery(queryString).setParameter(propertyName, value).getResultList();
    }

    @Override
    public List<T> findByProperty(String[] propertyNames, Object[] values) {
        if(propertyNames.length != values.length){
            throw new IllegalArgumentException("参数名的数量和参数值不匹配!propertyNames:" + propertyNames.length + ",values:" + values.length);
        }
        StringBuffer queryString = new StringBuffer();
        queryString.append("from " + persistentClass.getSimpleName() + " as model ");

        int size = propertyNames.length;
        if (size > 0) {
            queryString.append("where 1=1 ");
        }

        for (int i = 0; i < size; i++) {
            if(values[i] != null) {
                queryString.append(" and model." + propertyNames[i] + "= :" + propertyNames[i]);
            }
        }

        Query queryObject = entityManager.createQuery(queryString.toString());
        for (int i = 0; i < size; i++) {
            if(values[i] != null) {
                queryObject.setParameter(propertyNames[i], values[i]);
            }
        }

        return queryObject.getResultList();
    }

    @Override
    public List<T> findByProperty(String propertyName, Object value, String orderByStuff) {
        String queryString = "from " + persistentClass.getSimpleName() + " as model where model." + propertyName + "= :" + propertyName + " " + orderByStuff;
        return entityManager.createQuery(queryString).setParameter(propertyName, value).getResultList();
    }

    @Override
    public List<T> findByProperty(String[] propertyNames, Object[] values, String orderByStuff) {
        if(propertyNames.length != values.length){
            throw new IllegalArgumentException("参数名的数量和参数值不匹配!propertyNames:" + propertyNames.length + ",values:" + values.length);
        }
        StringBuffer queryString = new StringBuffer();
        queryString.append("from " + persistentClass.getSimpleName() + " as model ");

        int size = propertyNames.length;
        if (size > 0) {
            queryString.append("where 1=1 ");
        }

        for (int i = 0; i < size; i++) {
            if(values[i] != null) {
                queryString.append(" and model." + propertyNames[i] + "= :" + propertyNames[i]);
            }
        }

        queryString.append(" " + orderByStuff);

        Query queryObject = entityManager.createQuery(queryString.toString());
        for (int i = 0; i < size; i++) {
            if(values[i] != null) {
                queryObject.setParameter(propertyNames[i], values[i]);
            }
        }

        return queryObject.getResultList();
    }

    @Override
    public T getByProperty(String propertyName, Object value) {
        String queryString = "from " + persistentClass.getSimpleName() + " as model where model." + propertyName + "= :" + propertyName;
        try {
            return (T) entityManager.createQuery(queryString).setParameter(propertyName, value).getSingleResult();
        }catch(Exception e){
            return null;
        }
    }

    @Override
    public T getByProperty(String[] propertyNames, Object[] values) {
        if(propertyNames.length != values.length){
            throw new IllegalArgumentException("参数名的数量和参数值不匹配!propertyNames:" + propertyNames.length + ",values:" + values.length);
        }
        StringBuffer queryString = new StringBuffer();
        queryString.append("from " + persistentClass.getSimpleName() + " as model ");

        int size = propertyNames.length;
        if (size > 0) {
            queryString.append("where 1=1 ");
        }

        for (int i = 0; i < size; i++) {
            if(values[i] != null) {
                queryString.append(" and model." + propertyNames[i] + "= :" + propertyNames[i]);
            }
        }

        Query queryObject = entityManager.createQuery(queryString.toString());
        for (int i = 0; i < size; i++) {
            if(values[i] != null) {
                queryObject.setParameter(propertyNames[i], values[i]);
            }
        }

        try{
            return (T) queryObject.getSingleResult();
        }catch(Exception e){
            return null;
        }
    }

    @Override
    public int listCount() {
        Long count = entityManager.createQuery("select count(distinct model) from " + persistentClass.getName() + " as model",Long.class)
            .getSingleResult();
        return count == null ? 0 : count.intValue();
    }

    @Override
    public int findByPropertyCount(String propertyName, Object value) {
        String queryString = "select count(distinct model) from " + persistentClass.getName() + " as model where model." + propertyName + "= :" + propertyName;
        Long count = entityManager.createQuery(queryString, Long.class)
                .setParameter(propertyName, value)
                .getSingleResult();
        return count == null ? 0 : count.intValue();
    }

    @Override
    public int findByPropertyCount(String[] propertyNames, Object[] values) {
        if(propertyNames.length != values.length){
            throw new IllegalArgumentException("参数名的数量和参数值不匹配!propertyNames:" + propertyNames.length + ",values:" + values.length);
        }
        StringBuffer queryString = new StringBuffer();
        queryString.append("select count(distinct model) from " + persistentClass.getName() + " as model");

        int size = propertyNames.length;
        if (size > 0) {
            queryString.append(" where 1=1 ");
        }

        for (int i = 0; i < size; i++) {
            if(values[i] != null) {
                queryString.append(" and model." + propertyNames[i] + "= :" + propertyNames[i]);
            }
        }

        Query queryObject = entityManager.createQuery(queryString.toString(),Long.class);
        for (int i = 0; i < size; i++) {
            if(values[i] != null) {
                queryObject.setParameter(propertyNames[i], values[i]);
            }
        }

        Long count = (Long) queryObject.getSingleResult();
        return count == null ? 0 : count.intValue();
    }

    @Override
    public List<T> findByPropertyPage(String propertyName, Object value, String orderByStuff, Pager pager) {
        String queryString = "from " + persistentClass.getSimpleName() + " as model where model." + propertyName + "= :" + propertyName + " " + orderByStuff;
        return entityManager.createQuery(queryString)
                    .setParameter(propertyName, value)
                    .getResultList();
    }

    @Override
    public List<T> findByPropertyPage(String[] propertyNames, Object[] values, String orderByStuff, Pager pager) {
        if(propertyNames.length != values.length){
            throw new IllegalArgumentException("参数名的数量和参数值不匹配!propertyNames:" + propertyNames.length + ",values:" + values.length);
        }
        StringBuffer queryString = new StringBuffer();
        queryString.append("from " + persistentClass.getSimpleName() + " as model ");

        int size = propertyNames.length;
        if (size > 0) {
            queryString.append("where 1=1 ");
        }

        for (int i = 0; i < size; i++) {
            if(values[i] != null) {
                queryString.append(" and model." + propertyNames[i] + "= :" + propertyNames[i]);
            }
        }

        if(orderByStuff != null && orderByStuff.length() > 0){
            queryString.append(orderByStuff);
        }

        Query queryObject = entityManager.createQuery(queryString.toString());
        for (int i = 0; i < size; i++) {
            queryObject.setParameter(propertyNames[i], values[i]);
        }

        return queryObject.setFirstResult(pager.getStartRow()).setMaxResults(pager.getPageSize()).getResultList();
    }

    @Override
    public List<T> findByHql(String hql) {
        return entityManager.createQuery(hql).getResultList();
    }

    @Override
    public List<T> findByHqlWhere(String hqlWhere) {
        StringBuffer queryString = new StringBuffer();
        queryString.append("from " + persistentClass.getSimpleName() + " as model where 1=1 ");

        if (hqlWhere != null && hqlWhere.length() > 0) {
            queryString.append(hqlWhere);
        }

        return entityManager.createQuery(queryString.toString()).getResultList();
    }

    @Override
    public List<T> findByHqlWhere(String hqlWhere, String[] propertyNames, Object[] values, String orderByStuff) {
        if(propertyNames.length != values.length){
            throw new IllegalArgumentException("参数名的数量和参数值不匹配!propertyNames:" + propertyNames.length + ",values:" + values.length);
        }
        StringBuffer queryString = new StringBuffer();
        queryString.append("from " + persistentClass.getSimpleName() + " as model where 1=1 ");

        if (hqlWhere != null && hqlWhere.length() > 0) {
            queryString.append(hqlWhere);
        }

        int size = propertyNames.length;
        for (int i = 0; i < size; i++) {
            if(values[i] != null) {
                queryString.append(" and model." + propertyNames[i] + "= :" + propertyNames[i]);
            }
        }

        queryString.append("" + orderByStuff);

        Query queryObject = entityManager.createQuery(queryString.toString());
        for (int i = 0; i < size; i++) {
            if(values[i] != null) {
                queryObject.setParameter(propertyNames[i], values[i]);
            }
        }
        return queryObject.getResultList();
    }

    @Override
    public int findByHqlWhereCount(String hqlWhere, String[] propertyNames, Object[] values) {
        if(propertyNames.length != values.length){
            throw new IllegalArgumentException("参数名的数量和参数值不匹配!propertyNames:" + propertyNames.length + ",values:" + values.length);
        }
        StringBuffer queryString = new StringBuffer();
        queryString.append("select count(distinct model) from " + persistentClass.getSimpleName() + " as model where 1=1 ");

        if (hqlWhere != null && hqlWhere.length() > 0) {
            queryString.append(hqlWhere);
        }

        int size = propertyNames.length;
        for (int i = 0; i < size; i++) {
            if(values[i] != null) {
                queryString.append(" and model." + propertyNames[i] + "= :" + propertyNames[i]);
            }
        }

        Query queryObject = entityManager.createQuery(queryString.toString());
        for (int i = 0; i < size; i++) {
            if(values[i] != null) {
                queryObject.setParameter(propertyNames[i], values[i]);
            }
        }

        Long count = entityManager.createQuery(queryString.toString(),Long.class).getSingleResult();
        return count == null ? 0 : count.intValue();
    }

    @Override
    public List<T> findByHqlWherePage(String hqlWhere, Pager pager, String[] propertyNames, Object[] values, String orderByStuff) {
        if(propertyNames.length != values.length){
            throw new IllegalArgumentException("参数名的数量和参数值不匹配!propertyNames:" + propertyNames.length + ",values:" + values.length);
        }
        StringBuffer queryString = new StringBuffer();
        queryString.append("from " + persistentClass.getSimpleName() + " as model where 1=1 ");

        if (hqlWhere != null && hqlWhere.length() > 0) {
            queryString.append(hqlWhere);
        }

        int size = propertyNames.length;
        for (int i = 0; i < size; i++) {
            if(values[i] != null) {
                queryString.append(" and model." + propertyNames[i] + "= :" + propertyNames[i]);
            }
        }
        queryString.append(" " + orderByStuff);

        Query queryObject = entityManager.createQuery(queryString.toString()).setFirstResult(pager.getStartRow()).setMaxResults(pager.getPageSize());
        for (int i = 0; i < size; i++) {
            if(values[i] != null) {
                queryObject.setParameter(propertyNames[i], values[i]);
            }
        }

        return queryObject.getResultList();
    }

    @Override
    public List<T> findByHqlWherePage(String hqlWhere, Pager pager) {
        StringBuffer queryString = new StringBuffer();
        queryString.append("from " + persistentClass.getSimpleName() + " as model where 1=1 ");

        if (hqlWhere != null && hqlWhere.length() > 0) {
            queryString.append(hqlWhere);
        }

        return entityManager.createQuery(queryString.toString())
                    .setFirstResult(pager.getStartRow())
                    .setMaxResults(pager.getPageSize())
                    .getResultList();
    }

    @Override
    public int findByHqlWhereCount(String hqlWhere) {
        StringBuffer queryString = new StringBuffer();
        queryString.append("select count(distinct model) from " + persistentClass.getSimpleName() + " as model where 1=1 ");

        if (hqlWhere != null && hqlWhere.length() > 0) {
            queryString.append(hqlWhere);
        }

        Long count = entityManager.createQuery(queryString.toString(), Long.class).getSingleResult();
        return count == null ? 0 : count.intValue();
    }

    @Override
    public int findByHqlCount(String hql) {
        Long count = entityManager.createQuery(hql,Long.class).getSingleResult();
        return count == null ? 0 : count.intValue();
    }

    @Override
    public int findByHqlCount(String hql, String[] paramName, Object[] paramValue) {
        Query queryObject = entityManager.createQuery(hql,Integer.class);
        for(int i=0;i<paramName.length;i++){
            if(paramValue[i] != null) {
                queryObject.setParameter(paramName[i], paramValue[i].toString());
            }
        }

        return (int) queryObject.getSingleResult();
    }

    @Override
    public List<T> findByHqlPage(String hqlWhere, Pager pager) {
        StringBuffer queryString = new StringBuffer();
        queryString.append("from " + persistentClass.getSimpleName() + " as model where 1=1 ");

        if (hqlWhere != null && hqlWhere.length() > 0) {
            queryString.append(hqlWhere);
        }

        return entityManager.createQuery(queryString.toString())
                    .setFirstResult(pager.getStartRow())
                    .setMaxResults(pager.getPageSize())
                    .getResultList();
    }

    @Override
    public List<T> findByHqlPage(String hql, Pager pager, String[] paramName, Object[] paramValue) {
        Query queryObject = entityManager.createQuery(hql);
        for(int i=0;i<paramName.length;i++){
            if(paramValue[i] != null) {
                queryObject.setParameter(paramName[i], paramValue[i].toString());
            }
        }

        return queryObject.setFirstResult(pager.getStartRow())
                    .setMaxResults(pager.getPageSize())
                    .getResultList();
    }

    @Override
    public List findBySql(String sql) {
        return entityManager.createNativeQuery(sql).getResultList();
    }

    @Override
    public List<?> findBySql(String sql, Class entity) {
        return entityManager.createNativeQuery(sql,entity).getResultList();
    }

    @Override
    public List findBySqlPage(String sql, Pager pager) {
        return entityManager.createNativeQuery(sql)
                    .setFirstResult(pager.getStartRow())
                    .setMaxResults(pager.getPageSize())
                    .getResultList();
    }

    @Override
    public List<?> findBySqlPage(String sql, Class entity, Pager pager) {
        return entityManager.createNativeQuery(sql,entity)
                    .setFirstResult(pager.getStartRow())
                    .setMaxResults(pager.getPageSize())
                    .getResultList();
    }

    @Override
    public int executeSql(String sql) {
        return entityManager.createNativeQuery(sql).executeUpdate();
    }

    @Override
    public int findBySqlCount(String sql) {
        Long count = (Long) entityManager.createNativeQuery("select count(*) from ( " + sql + " ) as usertable",Long.class).getSingleResult();
        return count == null ? 0 : count.intValue();
    }

    @Override
    public double findBySqlSumDouble(String sql, String columnName) {
        return (double) entityManager.createNativeQuery("select sum(" + columnName + ") from ( " + sql + " ) as usertable",Double.class)
                    .getSingleResult();
    }

    @Override
    public double findByHqlDouble(String hql) {
        Double result = entityManager.createQuery(hql,Double.class).getSingleResult();
        return result == null ? 0 : result;
    }

    @Override
    public long findByHqlLong(String hql) {
        return entityManager.createQuery(hql,Long.class).getSingleResult();
    }

    @Override
    public List<T> findByHqlTopWhere(String hqlWhere, int topNum) {
        StringBuffer queryString = new StringBuffer();
        queryString.append("from " + persistentClass.getSimpleName() + " as model where 1=1 ");

        if (hqlWhere != null && hqlWhere.length() > 0) {
            queryString.append(hqlWhere);
        }
        return entityManager.createQuery(queryString.toString()).setFirstResult(0).setMaxResults(topNum).getResultList();
    }

    @Override
    public List<T> findColumnsByHql(String hql, String hqlWhere, String orderByStuff, Pager pager) {
        StringBuffer queryString = new StringBuffer();

        queryString.append(hql + " as model where 1=1 ");

        if (hqlWhere != null && hqlWhere.length() > 0) {
            queryString.append(hqlWhere);
        }

        queryString.append(" " + orderByStuff);

        return entityManager.createQuery(queryString.toString()).setFirstResult(pager.getStartRow()).setMaxResults(pager.getPageSize()).getResultList();
    }

    @Override
    public List<Object[]> findByHqlObjects(String hql) {
        return entityManager.createQuery(hql).getResultList();
    }

    @Override
    public List<Object[]> findByHqlObjects(String hql, String[] paramName, Object[] paramValue) {
        Query queryObject = entityManager.createQuery(hql);
        for(int i=0;i<paramName.length;i++){
            if(paramValue[i] != null) {
                queryObject.setParameter(paramName[i], paramValue[i].toString());
            }
        }

        return queryObject.getResultList();
    }

    @Override
    public List<Object[]> findByHqlPageObjects(String hql, Pager pager) {
        return entityManager.createQuery(hql).setFirstResult(pager.getStartRow()).setMaxResults(pager.getPageSize()).getResultList();
    }

    @Override
    public List<Object[]> findByHqlPageObjects(String hql, Pager pager, String[] paramName, Object[] paramValue) {
        Query queryObject = entityManager.createQuery(hql);
        for(int i=0;i<paramName.length;i++){
            if(paramValue[i] != null) {
                queryObject.setParameter(paramName[i], paramValue[i].toString());
            }
        }

        return queryObject.setFirstResult(pager.getStartRow()).setMaxResults(pager.getPageSize()).getResultList();
    }

    @Override
    public List<Object[]> findBySqlObjList(String sql) {
        return entityManager.createNativeQuery(sql).getResultList();
    }

    @Override
    public List<Object[]> findBySqlObjListByPager(String sql, Pager pager) {
        return entityManager.createNativeQuery(sql)
                    .setFirstResult(pager.getStartRow())
                    .setMaxResults(pager.getPageSize())
                    .getResultList();
    }

    /**
     * 保存对象
     * @param item 保存对象
     * @return
     */
    private T innerSave(T item) {
        try {
            if(item==null)return null;
            Class<?> clazz = item.getClass();

            //获取主键
            Field idField = clazz.getField("id");
            if(idField == null){
                idField = clazz.getField("primaryKey");
            }
            if(idField == null){
                // 遍历所有属性，以@Id声明确认主键
                Field[] fields = clazz.getFields();
                for(Field f : fields){
                    if(f.getAnnotation(Id.class) != null){
                        idField = f;
                        break;
                    }
                }
            }

            if(idField != null){
                Class<?> type = idField.getType();
                Object val = idField.get(item);
                if((type == long.class || type == Long.class) && (val == null || Long.parseLong(val.toString()) == 0)){
                    // long类型主键，以snowflake为主键
                    idField.set(item, snowflakeIdMaker.nextId());
                } else if(type == String.class && (val==null || "".equals(val))){
                    // String类型主键，以UUID为主键
                    idField.set(item, UUID.randomUUID().toString().replace("-", "").toLowerCase());
                }
            }

            entityManager.merge(item);
            entityManager.flush();
            if(idField!=null){
                return item;
            }else {
                return null;
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException(e);
        }
    }

    /************** 部分业务 *************/
    private String getNowTime(boolean needSecond) {
        return needSecond ? getNowTime("yyyy-MM-dd HH:mm:ss") : getNowTime("yyyy-MM-dd HH:mm");
    }

    private String getNowTime(String format) {
        String dateTime;
        try {
            dateTime = (new SimpleDateFormat(format)).format(new Date());
        } catch (Exception e) {
            dateTime = (new SimpleDateFormat("yyyy-MM-dd HH:mm:ss")).format(new Date());
        }

        return dateTime;
    }

    @Override
    public int deleteWeaklyById(ID id,String username) {
        return deleteWeaklyByProperty("id", id,username);
    }

    @Override
    public int deleteWeaklyByIds(Long[] ids,String username) {
        return deleteWeaklyByPropertyIn("id",  ids,username);
    }

    @Override
    public int deleteWeaklyByProperty(String propName, Object propValue,String username) {
        return deleteWeaklyByProperties(new String[] { propName }, new Object[] { propValue },username);
    }

    /**
     * 批量弱删除
     * @return
     */
    @Override
    public int deleteWeaklyByPropertyIn(String propName, Object[] propValues,String username) {
        if(propValues != null && propValues.length > 0){
            String hql = "update " + persistentClass.getName()
                    + " model set model.isDeleted = :isDeleted,model.modifyUser= :modifyUser,model.modifyDatetime=:modifyDatetime  where model."
                    + propName + " in(:"+propName+") ";
            Query query = entityManager.createQuery(hql);
            query.setParameter("isDeleted",CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
            query.setParameter("modifyUser",username);
            query.setParameter("modifyDatetime",getNowTime(true));
            query.setParameter(propName, Arrays.asList(propValues));

            return query.executeUpdate();
        }else{
            throw new IllegalArgumentException("弱删除失败："+propName+":" + propValues);
        }
    }

    /**
     * 批量弱删除
     * @return
     */
    @Override
    public int deleteWeaklyByProperties(String[] propName, Object[] propValue,String username) {
        if ((propName != null) && (propName.length > 0) && (propValue != null) && (propValue.length > 0) && (propValue.length == propName.length)) {
            StringBuffer sb = new StringBuffer("update " + persistentClass.getName()
                    + " model set model.isDeleted = :isDeleted,model.modifyUser= :modifyUser,model.modifyDatetime=:modifyDatetime where 1=1 ");
            appendQL(sb,propName,propValue);
            Query query = entityManager.createQuery(sb.toString());
            query.setParameter("modifyUser",username);
            query.setParameter("modifyDatetime",getNowTime(true));
            query.setParameter("isDeleted",CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());

            setParameter(query,propName,propValue);

            return query.executeUpdate();
        }else{
            throw new IllegalArgumentException("弱删除失败："+propName+":" + propValue);
        }
    }

    @Override
    public int updateValidStatusById(ID id,int status,String username) {
        return updateValidStatusByProperty("id", id,status,username);
    }

    @Override
    public int updateValidStatusByIds(Long[] ids,int status,String username) {
        return updateValidStatusByPropertyIn("id",  ids,status,username);
    }

    @Override
    public int updateValidStatusByProperty(String propName, Object propValue,int status,String username) {
        return updateValidStatusByProperties(new String[] { propName }, new Object[] { propValue },status,username);
    }

    /**
     * 批量禁用
     * @return
     */
    @Override
    public int updateValidStatusByPropertyIn(String propName, Object[] propValues,int status,String username) {
        if(propValues != null && propValues.length > 0){
            String hql = "update " + persistentClass.getName()
                    + " model set model.isValid = :isValid ,model.modifyUser= :modifyUser,model.modifyDatetime=:modifyDatetime where model."
                    + propName + " in(:"+propName+") ";
            Query query = entityManager.createQuery(hql);
            query.setParameter("modifyUser",username);
            query.setParameter("modifyDatetime",getNowTime(true));
            query.setParameter("isValid",status);
            query.setParameter(propName, Arrays.asList(propValues));

            return query.executeUpdate();
        }else{
            throw new IllegalArgumentException("修改状态失败："+propName+":" + propValues);
        }
    }

    /**
     * 批量禁用
     * @return
     */
    @Override
    public int updateValidStatusByProperties(String[] propName, Object[] propValue,int status,String username) {
        if ((propName != null) && (propName.length > 0) && (propValue != null) && (propValue.length > 0) && (propValue.length == propName.length)) {
            StringBuffer sb = new StringBuffer("update " + persistentClass.getName()
                    + " model set model.isValid = :isValid,model.modifyUser= :modifyUser,model.modifyDatetime=:modifyDatetime where 1=1 ");
            appendQL(sb,propName,propValue);
            Query query = entityManager.createQuery(sb.toString());
            query.setParameter("modifyUser",username);
            query.setParameter("modifyDatetime",getNowTime(true));
            query.setParameter("isValid",status);

            setParameter(query,propName,propValue);

            return query.executeUpdate();
        }else{
            throw new IllegalArgumentException("修改状态失败："+propName+":" + propValue);
        }
    }

    @Override
    public boolean isExitByProperty(String propertyName, Object value) {
        //大于0，存在
        return findByPropertyCount(propertyName,value) > 0;
    }

    @Override
    public boolean isExitByProperty(String[] propertyNames, Object[] values) {
        return findByPropertyCount(propertyNames, values) > 0;
    }

    @Override
    public boolean isExitByHqlWhere(String hqlWhere, String[] propertyNames, Object[] values) {
        return findByHqlWhereCount(hqlWhere,propertyNames,values) > 0;
    }

    @Override
    public boolean isExitByHqlWhere(String hqlWhere) {
        return findByHqlWhereCount(hqlWhere) > 0;
    }

    @Override
    public boolean isExitByHql(String hql) {
        return findByHqlCount(hql) > 0;
    }

    @Override
    public boolean isExitByHql(String hql, String[] paramName, Object[] paramValue) {
        return findByHqlCount(hql) > 0;
    }

    @Override
    public boolean isExitBySql(String sql) {
        return findBySqlCount(sql) > 0;
    }
}
